@@ -30,6 +30,10 @@
 //! For the previous example it would be sufficient to use one builder per glob and to specify
 //! the root folder when building the pattern (see examples below).
 //!
+//! # Globs
+//!
+//! Please check the documentation of [globset][globset] for the available glob format.
+//!
 //! # Example: A simple match.
 //!
 //! The following example uses the files stored in the `test-files` folder, we're trying to match
@@ -37,6 +41,21 @@
 //! relative path component).
 //!
 //! ```
+//! /*
+//!     Example files:
+//!     globmatch/test-files/.hidden
+//!     globmatch/test-files/.hidden/h_1.txt
+//!     globmatch/test-files/.hidden/h_0.txt
+//!     globmatch/test-files/a/a2/a2_0.txt
+//!     globmatch/test-files/a/a0/a0_0.txt
+//!     globmatch/test-files/a/a0/a0_1.txt
+//!     globmatch/test-files/a/a0/A0_3.txt
+//!     globmatch/test-files/a/a0/a0_2.md
+//!     globmatch/test-files/a/a1/a1_0.txt
+//!     globmatch/test-files/some_file.txt
+//!     globmatch/test-files/b/b_0.txt
+//!  */
+//!
 //! use globmatch;
 //!
 //! # fn example_a() -> Result<(), String> {
@@ -140,7 +159,6 @@
 #[cfg(doctest)]
 doc_comment::doctest!("../readme.md");
 
-use globset;
 use std::path;
 
 mod error;
@@ -189,17 +207,17 @@ impl<'a> Builder<'a> {
     /// The actual facade for `globset::Glob`.
     #[doc(hidden)]
     fn glob_for(&self, glob: &str) -> Result<globset::Glob, String> {
-        Ok(globset::GlobBuilder::new(glob)
+        globset::GlobBuilder::new(glob)
             .literal_separator(REQUIRE_PATHSEP)
             .case_insensitive(!self.case_sensitive)
             .build()
             .map_err(|err| {
                 format!(
                     "'{}': {}",
-                    self.glob.to_string(),
+                    self.glob,
                     utils::to_upper(err.kind().to_string())
                 )
-            })?)
+            })
     }
 
     /// Builds a [`Matcher`] for the given [`Builder`] relative to `root`.
@@ -244,7 +262,7 @@ impl<'a> Builder<'a> {
     /// This [`Glob`] that can be used for filtering paths provided by a [`Matcher`] (created
     /// using the `build` function).
     pub fn build_glob(&self) -> Result<Glob<'a>, String> {
-        if self.glob.len() == 0 {
+        if self.glob.is_empty() {
             return Err("Empty glob".to_string());
         }
 
@@ -263,7 +281,7 @@ impl<'a> Builder<'a> {
     /// Yes, it would be sufficient to use the pattern `**/pattern` in the first place. This is
     /// a simple commodity function.
     pub fn build_glob_set(&self) -> Result<GlobSet<'a>, String> {
-        if self.glob.len() == 0 {
+        if self.glob.is_empty() {
             return Err("Empty glob".to_string());
         }
 
@@ -281,7 +299,7 @@ impl<'a> Builder<'a> {
             .map_err(|err| {
                 format!(
                     "'{}': {}",
-                    self.glob.to_string(),
+                    self.glob,
                     utils::to_upper(err.kind().to_string())
                 )
             })?;
@@ -301,9 +319,12 @@ pub struct Matcher<'a, P>
 where
     P: AsRef<path::Path>,
 {
-    glob: &'a str, // Original glob-pattern
-    root: P,       // Root path of a resolved pattern
-    rest: &'a str, // Remaining pattern after root has been resolved
+    glob: &'a str,
+    /// Original glob-pattern
+    root: P,
+    /// Root path of a resolved pattern
+    rest: &'a str,
+    /// Remaining pattern after root has been resolved
     matcher: globset::GlobMatcher,
 }
 
@@ -314,7 +335,7 @@ where
     type Item = Result<path::PathBuf, Error>;
     type IntoIter = IterAll<P>;
 
-    /// Transform the `Matcher` into a recursive directory iterator.
+    /// Transform the [`Matcher`] into a recursive directory iterator.
     fn into_iter(self) -> Self::IntoIter {
         let walk_root = path::PathBuf::from(self.root.as_ref());
         IterAll::new(
@@ -329,34 +350,52 @@ impl<'a, P> Matcher<'a, P>
 where
     P: AsRef<path::Path>,
 {
+    /// Provides the original glob-pattern used to create this [`Matcher`].
+    ///
+    /// This is the unchanged glob, i.e., no relative path components have been resolved.
     pub fn glob(&self) -> &str {
         self.glob
     }
 
+    /// Provides the resolved root folder used by the [`Matcher`].
+    ///
+    /// This directory already contains the path components from the original glob. The main
+    /// intention of this function is to for debugging or logging (thus a String).
     pub fn root(&self) -> String {
         let path = path::PathBuf::from(self.root.as_ref());
         String::from(path.to_str().unwrap())
     }
 
+    /// Provides the resolved glob used by the [`Matcher`].
+    ///
+    /// All relative path components have been resolved for this glob. The glob is of type &str
+    /// since all globs are input parameters and specified as strings (and not paths).
     pub fn rest(&self) -> &str {
         self.rest
     }
 
+    /// Checks whether the provided path is a match for the stored glob.
     pub fn is_match(&self, p: P) -> bool {
         self.matcher.is_match(p)
     }
 }
 
+/// Wrapper type for glob matching.
+///
+/// This type is created by [`Builder::build_glob`] for a single glob on which no transformations
+/// or path resolutions have been performed.
 pub struct Glob<'a> {
     glob: &'a str,
     pub matcher: globset::GlobMatcher,
 }
 
 impl<'a> Glob<'a> {
+    /// Provides the original glob-pattern used to create this [`Glob`].
     pub fn glob(&self) -> &str {
         self.glob
     }
 
+    /// Checks whether the provided path is a match for the stored glob.
     pub fn is_match<P>(&self, p: P) -> bool
     where
         P: AsRef<path::Path>,
@@ -365,16 +404,23 @@ impl<'a> Glob<'a> {
     }
 }
 
+/// Comfort type for glob matching.
+///
+/// This type is created by [`Builder::build_glob_set`] (refer to the function documentation). The
+/// matcher stores two globs created from the original pattern as `[**/pattern, pattern]` for
+/// easy matching on multiple paths.
 pub struct GlobSet<'a> {
     glob: &'a str,
     pub matcher: globset::GlobSet,
 }
 
 impl<'a> GlobSet<'a> {
+    /// Provides the original glob-pattern used to create this [`GlobSet`].
     pub fn glob(&self) -> &str {
         self.glob
     }
 
+    /// Checks whether the provided path is a match for any of the two stored globs.
     pub fn is_match<P>(&self, p: P) -> bool
     where
         P: AsRef<path::Path>,
@@ -415,7 +461,7 @@ mod tests {
         // function declaration within function. yay this starts to feel like python :D
         fn match_glob<'a>(f: &'a str, m: &globset::GlobMatcher) -> Option<&'a str> {
             match m.is_match(f) {
-                true => Some(f.as_ref()),
+                true => Some(f),
                 false => None,
             }
         }
@@ -432,12 +478,11 @@ mod tests {
                 .compile_matcher())
         }
 
-        fn test_for(glob: &str, len: usize, files: &Vec<&str>, case_sensitive: bool) {
+        fn test_for(glob: &str, len: usize, files: &[&str], case_sensitive: bool) {
             let glob = glob_for(glob, case_sensitive).unwrap();
             let matches = files
                 .iter()
-                .map(|f| match_glob(f, &glob))
-                .flatten()
+                .filter_map(|f| match_glob(f, &glob))
                 .collect::<Vec<_>>();
             println!(
                 "matches for {}:\n'{}'",
@@ -510,7 +555,7 @@ mod tests {
     some helper functions for testing
     */
 
-    fn log_paths<P>(paths: &Vec<P>)
+    fn log_paths<P>(paths: &[P])
     where
         P: AsRef<path::Path>,
     {
@@ -524,7 +569,7 @@ mod tests {
         );
     }
 
-    fn log_paths_and_assert<P>(paths: &Vec<P>, expected_len: usize)
+    fn log_paths_and_assert<P>(paths: &[P], expected_len: usize)
     where
         P: AsRef<path::Path>,
     {
@@ -639,13 +684,8 @@ mod tests {
 
     #[test]
     fn match_flavours() -> Result<(), String> {
-        // TODO: continue here for different relative pattern styles
-        // TODO: the util function checks that there are no relative parts in the REMAINDER
+        // TODO: implememnt tests for different relative pattern styles
+        // TODO: also provide failing tests for relative parts in the rest/remainder glob
         Ok(())
     }
 }
-
-// TODO: checkout coverage
-// https://github.com/mozilla/grcov
-// https://marco-c.github.io/2020/11/24/rust-source-based-code-coverage.html
-// https://github.com/marco-c/rust-code-coverage-sample/blob/main/run_gcov.sh
